program hplxfile;

{
 ***** version 1.0 ***** 
 (c) Pavel Zampach (zampach@volny.cz), 2003
}

{$APPTYPE CONSOLE}
{$IOChecks Off}
{$RangeChecks On}

uses
  Windows,
  SysUtils,
  Filer in 'FILER.PAS';

var
  FileName1, FileName2, Par, CPort : string;
  i : byte;
  CSpeed : longword;
  Command, CP : char;
  DoneOK, Move : boolean;

{ ------------------------------------------------------------------ }

function trimFileName (const OldName : string) : string;

const
  DOSChar : set of char = ['A'..'Z', 'a'..'z', '0'..'9', '_', '-', '~'];

var
  i, q, PosPoint : integer;
  c : char;

begin
  Result := '';
  PosPoint := lastDelimiter ('.', OldName);
  if PosPoint = 0 then
    PosPoint := length (OldName) + 1;

  for i := 1 to PosPoint - 1 do begin
    c := OldName[i];
    if (c in DOSChar) and (length (Result) < 8) then
      Result := Result + c;
  end;

  Result := Result + '.';
  q := length (Result) + 3;

  for i := (PosPoint + 1) to (length (OldName)) do begin
    c := OldName[i];
    if (c in DOSChar) and (length (Result) < q) then
      Result := Result + c;
  end;
end;

{ ------------------------------------------------------------------ }

function getDirEntry (const RemoteName : pchar; var dirEntry : TDirEntry) : boolean;

begin
  Result := false;
  
  if not FilerRequest (CMD_ASK_DIR, RemoteName) then begin
    FilerReset;
    exit;
  end;
  Packet.data := @dirEntry;
  if not FilerRequest (CMD_GET_DIR) then begin
    FilerReset;
    exit;
  end;
  
  Result := true;
end;

{ ------------------------------------------------------------------ }

function HFDeleteFile (FileName : pchar) : boolean;

begin
  Result := FilerRequest (CMD_DEL_FILE, FileName);
  if not Result then FilerReset;
end;

{ ------------------------------------------------------------------ }

function HFMkDir (DirName : pchar) : boolean;

begin
  Result := FilerRequest (CMD_MAKE_DIR, DirName);
  if not Result then FilerReset;
end;

{ ------------------------------------------------------------------ }

function HFRemoveDir (DirName : pchar) : boolean;
  
begin
  Result := FilerRequest (CMD_DEL_DIR, DirName);
  if not Result then FilerReset;
end;

{ ------------------------------------------------------------------ }

function HFRenMovFile (OldName, NewName : pchar) : boolean;

begin
  Packet.size2 := strLen (NewName);
  Packet.data2 := TPData (NewName);
  Result := FilerRequest (CMD_REN_FILE, OldName);
  if not Result then FilerReset;
end;
  
{ ------------------------------------------------------------------ }

function HFGetFile (RemoteName, LocalName : pchar; Move :boolean) : boolean;

var
  Delivered : longword;
  WorkFile : file;
  PBuffer  : TPData;

begin
  Result := false;
  
  if not FilerRequest (CMD_GET_FILENAME, RemoteName) then begin
    FilerReset;
    exit;
  end;

  if not FilerRequest (CMD_INIT_GET) then begin
    FilerReset;
    exit;
  end;

  assignFile (WorkFile, strPas (LocalName));
  rewrite (WorkFile, 1);
  if IOResult <> 0 then begin
    FilerReset;
    exit;
  end;

  new (PBuffer);

  try
    repeat           // ------- main copy loop -------
      Packet.size := PACKET_DATA_SIZE;
      Packet.data := PBuffer;
      if not FilerRequest (CMD_GET_DATA) then begin
        closeFile (WorkFile);
        FilerReset;
        FilerRequest (CMD_DATA_END);
        exit;
      end;

      Delivered   := Packet.size;

      BlockWrite (WorkFile, PBuffer^, Delivered);
      if IOResult <> 0 then begin
        closeFile (WorkFile);
        FilerReset;
        FilerRequest (CMD_DATA_END);
        exit;
      end;

    write ('.');

    until Delivered < PACKET_DATA_SIZE;   // ------- end of main copy loop -------

    closeFile (WorkFile);
    if IOResult <> 0 then begin
      FilerReset;
      exit;
    end;

    if not FilerRequest (CMD_DATA_END) then begin
      FilerReset;
      exit;
    end;

    if Move then
      if not FilerRequest (CMD_DEL_FILE, RemoteName) then begin
        FilerReset;
        exit;
      end;

    Result := true;

  finally
    dispose (PBuffer);
  end;
end;

{ ------------------------------------------------------------------ }

function HFPutFile (LocalName, RemoteName : pchar; Move : boolean) : boolean;

var
  Delivered   : longword;
  WorkFile    : file;
  PBuffer     : TPData;
  dirEntry    : TDirEntry;
  SRemoteName : string;

begin
  Result := false;
  
  SRemoteName := RemoteName;
  SRemoteName := extractFilePath (SRemoteName) + trimFileName (extractFileName (SRemoteName));

     { does remote file exist? }
  if not getDirEntry (RemoteName, DirEntry) then
    exit;
  if not Packet.endOfDir then
    if not FilerRequest (CMD_DEL_FILE, RemoteName) then begin
      FilerReset;
      exit;
    end;

  if not FilerRequest (CMD_SEND_FILENAME, RemoteName) then begin
    FilerReset;
    exit;
  end;

  assignFile (WorkFile, strpas (LocalName));
  reset (WorkFile, 1);
  if IOResult <> 0 then begin
    FilerReset;
    exit;
  end;

  new (PBuffer);

  try
    repeat // --------- main copy loop -----------
      BlockRead (WorkFile, PBuffer^, PACKET_DATA_SIZE, Delivered);
      if IOResult <> 0 then begin
        closeFile (WorkFile);
        FilerReset;
        FilerRequest (CMD_DATA_END);
        exit;
      end;

      Packet.size := Delivered;
      Packet.data := PBuffer;
      if not FilerRequest (CMD_SEND_DATA) then begin
        closeFile (WorkFile);
        FilerReset;
        FilerRequest (CMD_DATA_END);
        exit;
      end;
    write ('.');
    until Delivered < PACKET_DATA_SIZE; // end of main copy loop

    closeFile (WorkFile);
    if IOResult <> 0 then begin
      FilerReset;
      exit;
    end;

    if not FilerRequest (CMD_DATA_END) then begin
      FilerReset;
      exit;
    end;

    if Move then
      if not deleteFile (LocalName) then
        exit;

    Result := true;

  finally
    dispose (PBuffer);
  end;
end;

{ ------------------------------------------------------------------ }

begin   // main

// initialization
  FileName1 := '';
  FileName2 := '';
  CPort := 'COM1';
  CSpeed := 115200;
  Command := #0;
  DoneOK := false;
  Move := false;
 
 // command line processing
  for i := 1 TO ParamCount do begin
    Par := ParamStr (i);
    CP  := upCase(Par[2]);
    if (Par[1]) = '/' then
      case CP of
        'S' : CSpeed := strToInt (copy (Par, 3, 6));
        'P' : CPort := 'COM' + Par[3];
        'D', 'U' : begin
                     Command := CP;
                     Move := (length (Par) > 2) and (upCase (Par[3]) = 'M') ;
                   end;
        'L', 'R', 'M', 'N' : Command := CP;
      end
    else begin
      if FileName1 = '' then
        FileName1 := Par
      else  
        FileName2 := Par;
    end;
  end;
  
  if (Command = #0) or (FileName1 = '') then
    Halt (1);
  
  if not FilerConnect (CPort, CSpeed) then
    Halt (2);
    
// execution
  case Command of    
    'L' : DoneOK := HFDeleteFile (pchar (FileName1));
    'R' : DoneOK := HFRemoveDir (pchar (FileName1));
    'M' : DoneOK := HFMkDir (pchar (FileName1));
    'N' : DoneOK := HFRenMovFile (pchar (FileName1), pchar (FileName2));
    'D' : begin
            write ('Download ');
            DoneOK := HFGetFile (pchar (FileName1), pchar (FileName2), Move);
            writeln;
          end;
    'U' : begin
            write ('Upload ');
            DoneOK := HFPutFile (pchar (FileName1), pchar (FileName2), Move);
            writeln;
          end;
  end;
  
  FilerDisconnect;
  
  if not DoneOK then
    Halt (3);

end.